@@ -62,7 +62,6 @@ G_LOCK_DEFINE_STATIC (fam_connection);
 typedef struct {
 	FAMRequest request;
 	GnomeVFSURI *uri;
-	gboolean     cancelled;
 } FileMonitorHandle;
 
 #endif
@@ -106,7 +105,7 @@ GET_PATH_MAX (void)
 #endif
 
 static gchar *
-get_path_from_uri (GnomeVFSURI const *uri)
+get_path_from_uri (GnomeVFSURI *uri)
 {
 	gchar *path;
 
@@ -126,7 +125,7 @@ get_path_from_uri (GnomeVFSURI const *uri)
 }
 
 static gchar *
-get_base_from_uri (GnomeVFSURI const *uri)
+get_base_from_uri (GnomeVFSURI *uri)
 {
 	gchar *escaped_base, *base;
 
@@ -190,8 +189,8 @@ do_open (GnomeVFSMethod *method,
 	}
 
 	if (! (mode & GNOME_VFS_OPEN_RANDOM) && (mode & GNOME_VFS_OPEN_WRITE))
-		unix_mode |= O_TRUNC;
-	
+		mode |= O_TRUNC;
+
 	file_name = get_path_from_uri (uri);
 	if (file_name == NULL)
 		return GNOME_VFS_ERROR_INVALID_URI;
@@ -667,7 +666,7 @@ get_stat_info (GnomeVFSFileInfo *file_info,
 			                /* if we had an earlier ELOOP, don't get in an infinite loop here */
 			        || recursive
 					/* we don't care to follow links */
-				|| lstat (symlink_name, statptr) != 0
+				|| lstat (file_info->symlink_name, statptr) != 0
 					/* we can't make out where this points to */
 				|| !S_ISLNK (statptr->st_mode)) {
 					/* the next level is not a link */
@@ -872,43 +871,15 @@ do_get_file_info_from_handle (GnomeVFSMethod *method,
 	return GNOME_VFS_OK;
 }
 
-GHashTable *fstype_hash = NULL;
-G_LOCK_DEFINE_STATIC (fstype_hash);
-extern char const *filesystem_type (char *path, char *relpath, struct stat *statp);
 
 static gboolean
 do_is_local (GnomeVFSMethod *method,
 	     const GnomeVFSURI *uri)
 {
-	gchar *path;
-	gpointer local = NULL;
-
 	g_return_val_if_fail (uri != NULL, FALSE);
 
-	path = get_path_from_uri (uri);
-	if (path == NULL)
-		return TRUE; /* GNOME_VFS_ERROR_INVALID_URI */
-
-	G_LOCK (fstype_hash);
-	if (fstype_hash == NULL)
-		fstype_hash = g_hash_table_new_full (
-			g_str_hash, g_str_equal, g_free, NULL);
-	else
-		local = g_hash_table_lookup (fstype_hash, path);
-
-	if (local == NULL) {
-		struct stat statbuf;
-		if (stat (path, &statbuf) == 0) {
-			char const *type = filesystem_type (path, path, &statbuf);
-			gboolean is_local = strcmp (type, "nfs") && strcmp (type, "afs");
-			local = GINT_TO_POINTER (is_local ? 1 : -1);
-			g_hash_table_insert (fstype_hash, path, local);
-		}
-	} else
-		g_free (path);
-
-	G_UNLOCK (fstype_hash);
-	return GPOINTER_TO_INT (local) > 0;
+	/* We are always a native filesystem.  */
+	return TRUE;
 }
 
 
@@ -1659,11 +1630,6 @@ do_find_directory (GnomeVFSMethod *method,
 
 	switch (kind) {
 	case GNOME_VFS_DIRECTORY_KIND_TRASH:
-		/* Use 0700 (S_IRWXU) for the permissions,
-		 * regardless of the requested permissions, so other
-		 * users can't view the trash files.
-		 */
-		permissions = S_IRWXU;	
 		if (near_item_stat.st_dev != home_volume_stat.st_dev) {
 			/* This volume does not contain our home, we have to find/create the Trash
 			 * elsewhere on the volume. Use a heuristic to find a good place.
@@ -2032,14 +1998,14 @@ fam_callback (GIOChannel *source,
 	      GIOCondition condition,
 	      gpointer data)
 {
+	FileMonitorHandle *handle;
+	GnomeVFSURI *info_uri = NULL;
+	GnomeVFSMonitorEventType event_type = GNOME_VFS_MONITOR_EVENT_CHANGED;
+
 	G_LOCK (fam_connection);
 
 	while (FAMPending(fam_connection)) {
 		FAMEvent ev;
-		FileMonitorHandle *handle;
-		gboolean cancelled;
-		GnomeVFSMonitorEventType event_type;
-
 		if (FAMNextEvent(fam_connection, &ev) != 1) {
 			FAMClose(fam_connection);
 			g_free(fam_connection);
@@ -2049,8 +2015,9 @@ fam_callback (GIOChannel *source,
 		}
 
 		handle = (FileMonitorHandle *)ev.userdata;
-		cancelled = handle->cancelled;
-		event_type = -1;
+
+		info_uri = gnome_vfs_uri_append_file_name (handle->uri, 
+				ev.filename);
 
 		switch (ev.code) {
 			case FAMChanged:
@@ -2060,47 +2027,29 @@ fam_callback (GIOChannel *source,
 				event_type = GNOME_VFS_MONITOR_EVENT_DELETED;
 				break;
 			case FAMStartExecuting:
-				event_type = GNOME_VFS_MONITOR_EVENT_STARTEXECUTING;
+				event_type = 
+					GNOME_VFS_MONITOR_EVENT_STARTEXECUTING;
 				break;
 			case FAMStopExecuting:
-				event_type = GNOME_VFS_MONITOR_EVENT_STOPEXECUTING;
+				event_type = 
+					GNOME_VFS_MONITOR_EVENT_STOPEXECUTING;
 				break;
 			case FAMCreated:
 				event_type = GNOME_VFS_MONITOR_EVENT_CREATED;
 				break;
 			case FAMAcknowledge:
-				if (handle->cancelled) {
-					gnome_vfs_uri_unref (handle->uri);
-					g_free (handle);
-				}
-				break;
 			case FAMExists:
 			case FAMEndExist:
 			case FAMMoved:
-				/* Not supported */
+				event_type = -1;
 				break;
 		}
 
-		if (event_type != -1 && !cancelled) {
-			GnomeVFSURI *info_uri;
-			gchar *info_str;
 
-			/* 
-			 * FAM can send events with either a absolute or
-			 * relative (from the monitored URI) path, so check if
-			 * the filename starts with '/'.  
-			 */
-			if (ev.filename[0] == '/') {
-				info_str = gnome_vfs_get_uri_from_local_path (ev.filename);
-				info_uri = gnome_vfs_uri_new (info_str);
-				g_free (info_str);
-			} else
-				info_uri = gnome_vfs_uri_append_file_name (handle->uri, ev.filename);
-
-			gnome_vfs_monitor_callback ((GnomeVFSMethodHandle *)handle,
-						    info_uri, 
-						    event_type);
-			gnome_vfs_uri_unref (info_uri);
+		if (event_type != -1) {
+			gnome_vfs_monitor_callback (
+					(GnomeVFSMethodHandle *)handle,
+					info_uri, event_type);
 		}
 	}
 
@@ -2157,7 +2106,6 @@ do_monitor_add (GnomeVFSMethod *method,
 
 	handle = g_new0 (FileMonitorHandle, 1);
 	handle->uri = uri;
-	handle->cancelled = FALSE;
 	gnome_vfs_uri_ref (uri);
 	filename = get_path_from_uri (uri);
 	
@@ -2194,14 +2142,14 @@ do_monitor_cancel (GnomeVFSMethod *method,
 		return GNOME_VFS_ERROR_NOT_SUPPORTED;
 	}
 
-	if (handle->cancelled)
-		return GNOME_VFS_OK;
-
-	handle->cancelled = TRUE;
 	G_LOCK (fam_connection);
 	FAMCancelMonitor (fam_connection, &handle->request);
 	G_UNLOCK (fam_connection);
 
+	gnome_vfs_uri_unref (handle->uri);
+
+	g_free (handle);
+
 	return GNOME_VFS_OK;
 #else
 	return GNOME_VFS_ERROR_NOT_SUPPORTED;
